#include "FStrPairList.H"
#include <unistd.h>
#include <sys/stat.h>

#define FSL_DELIM ':'

FStrPairList::FStrPairList(const string &fname, bool autoupdate):
		StrPairList(),
		_autoupdate(autoupdate)  {

	_filename=fname;
	if ((_fp=fopen(_filename.c_str(),"r+"))!=NULL) {
		reread();
	}
	else
		_fp=fopen(_filename.c_str(),"w");
	if (_autoupdate) {
		struct stat fbuf;	
		stat(_filename.c_str(),&fbuf);
		_mtime=fbuf.st_mtime;
	}
	setbuf(_fp,NULL);
}

FStrPairList::~FStrPairList() {
	fclose(_fp);
}

int FStrPairList::add(const string &key, const string &data) {
//	fprintf(_fp,"%s%c%s\n",key.c_str(),FSL_DELIM,data.c_str());
	if (_autoupdate)
		update();
	StrPairList::add(key,data);
	rewrite();
	return 0;
}

int FStrPairList::remove(const string &key) {
	if (_autoupdate)
		update();
	StrPairList::remove(key);
	rewrite();
	return 0;
}

bool FStrPairList::inlist(const string &key) {
	if (_autoupdate)
		update();
	return (StrPairList::inlist(key));
}

int FStrPairList::get(const string &key, string &data) {
	if (_autoupdate)
		update();
	return (StrPairList::get(key,data));
}

int FStrPairList::get(unsigned int idx, string &key, string &data) {
	if (_autoupdate)
		update();
	return (StrPairList::get(idx,key,data));
}

string FStrPairList::operator[](unsigned int idx) {
	if (_autoupdate)
		update();
	return (StrPairList::operator[](idx));
}

unsigned int FStrPairList::size() {
	if (_autoupdate)
		update();
	return (StrPairList::size());
}

void FStrPairList::rewrite() {
	string nkey, ndata;
	rewind(_fp);
	// HACK alert !!!
	truncate(_filename.c_str(),0);
	for (unsigned int i=0; i < size(); i++) {
		StrPairList::get(i,nkey,ndata);
		fprintf(_fp,"%s%c%s\n",nkey.c_str(),FSL_DELIM,ndata.c_str());
	}
}

void FStrPairList::reread() {
	string
		key,
		data;
	char
		*ptr,
		buf[1024];

	StrPairList::clear();
	rewind(_fp);
	while (fgets(buf, sizeof(buf), _fp)) {
		buf[strlen(buf)-1]=0;
		if ((ptr=strchr(buf,FSL_DELIM))==NULL)
			continue;
		*ptr=0;
		key=buf;
		data=++ptr;
		StrPairList::add(key,data);
	}
}

bool FStrPairList::update() {
	struct stat buf;	
	stat(_filename.c_str(),&buf);
	if (_mtime < (unsigned)buf.st_mtime) {
		reread();
		_mtime=buf.st_mtime;
		return true;
	}
	return false;
}
